function P = autom_product(T,B)
%performs the product of 2 finite automata, with costs on transitions
%first automaton (T) is in fact a finite transition system with observables
%(a state has ONLY one observable - a label from set of all possible observables)
%second one (B) is one with guards on transitions (in this case a Buchi automata)
%a guard can be any combination of observables (as they would have "or" between them)
%(the alphabet of B (set of guards) is (must be) included (or equal) in the set of observations of T)
%T and B are implemented as structures

%T has fields Q, q0 (singleton), obs (indices of observables), adj (transitions with their costs)
%B has fields S, S0, F, trans
%P is the product automaton
%transitions of P (P.trans matrix) will inherit costs from corresponding transitions of T

%a dummy initial state will be temporarly added to T (in order to check if parts of formula are verified (or whole formula is false) from start)
%the dummy state has transitions only to initial state of T and no state has transitions to it
%the dummy state is the last (has the biggest label), has no observable and no vertices
%when a run is searched in the product automaton, the first state (the dummy one) will be removed

st_no_T=length(T.Q);    %number of states of T
T.Q=[T.Q st_no_T+1];    %dummy state
T.adj(st_no_T+1,T.q0)=1;    %add transitions to initial state
T.adj(:,st_no_T+1)=0;   %no state can transit in the dummy one
T.q0=st_no_T+1; %new initial state
P.S=cartesian_product(T.Q,B.S);  %states of product (row i gives on position 1 state from T, on position 2 state of B)
%P.S=mat2cell(states,ones(1,size(states,1)),2);  %cell for easy indexing (S(i) gives state i from the cartesian product)
init_st=cartesian_product(T.q0,B.S0);  %initial states of product
fin_st=cartesian_product(T.Q(1:st_no_T),B.F);  %final (accepting) states of product; the dummy state is not considered here

P.S0=[];    %row vectors to store indices of initial/final states
P.F=[];
for i=1:size(init_st,1) %indices (labels) of initial states
    P.S0=[P.S0 find((P.S(:,1)==init_st(i,1)) & (P.S(:,2)==init_st(i,2)))];
end
for i=1:size(fin_st,1) %find indices (labels) of final states
    P.F=[P.F find((P.S(:,1)==fin_st(i,1)) & (P.S(:,2)==fin_st(i,2)))];
end

P.trans=sparse(size(P.S,1),size(P.S,1));    %sparse matrix to keep possible transitions (trans(i,j)=cost, ~=0 if S(i,:) can transit in S(j,:))
for i=1:size(P.S,1) %search for possible transitions from current state (transitions enabled by T & B, as described in product of automata)
    tr_q=find(T.adj(P.S(i,1),:));  %labels of states of T in which q_i can transit
    tr_s=find(~cellfun('isempty',B.trans(P.S(i,2),:))); %indices (states) of B in which s_i can transit (for some predicates(observables))
    for j=1:length(tr_q)
        for k=1:length(tr_s)
            if ismember(T.obs(tr_q(j)), B.trans{P.S(i,2),tr_s(k)})    %the observable of q_j is in set that enables transition to s_k in Buchi
                ind=find((P.S(:,1)==tr_q(j)) & (P.S(:,2)==tr_s(k)));  %index of state [tr_q(j) tr_s(k)] (in which current state of P can transit)
                % P.trans(i,ind)=1;
                P.trans(i,ind)=T.adj(P.S(i,1),tr_q(j)); %cost for transition in P is equal with cost for transition q_i->q_j in T
            end
        end
    end
end
