function run_Tp = find_accepted_run(P)
%transitions can have different costs, not only adjacency is captured (as in an old function with same name)
%find an accepted run of product automaton P and project it to states of Tp (we need a run for Tp satisfying the LTL formula)
%(see definition of an accepted run: start from an initial state and visit infinitely often the set of final states)
%run_Tp will be a cell array with 2 elements: first (prefix) is a vector starting from initial state and containing a run (up to a final state, fs),
%second (suffix) is a vector containing a path which must be repeated infinitely (it starts with a neighbor of fs - last state from prefix - and it ends with fs)
%(if the suffix has just an element (fs), then fs has a loop in itself, which will be repeated infinitely often)
%the returned run_Tp will be the run with the smallest cost (shortest) prefix and smallest cost (shortest) suffix for that prefix

run_P=cell(1,length(P.S0)); %initialize with empty runs for each initial state of P; each element will be either empty or a cell with 2 elements (prefix & suffix)
r_cost=Inf(1,length(P.S0)); %cost of prefix+suffix for each run
for i=1:length(P.S0)    %find shortest run for P (if possible) for each initial state (P can have more initial states-same no. as Buchi; T has only one dummy init state, see autom_prod): find prefix and sufix for P
    [prefix, p_cost] = find_paths(P.trans, P.S0(i), P.F);   %find paths from each initial state of P to all final states
    %prefix = prefix( find( ~cellfun('isempty',prefix)));    %elimintate empty prefixes
    empt=find(p_cost==Inf); %indices of infinite costs (corresponding to empty prefixes) (above line worked if we don't take into account the cost)
    prefix(empt)=[];    %remove empty prefixes and their corresponding costs
    p_cost(empt)=[];
    if ~isempty(prefix) %there exists at least one non-empty prefix
        %[sh_p ind] = sort( cellfun('length',prefix));    %we need the order from shortest to longest prefix (we don't need sorted vector sh_p)
        [sh_p ind]=sort(p_cost);    %ascending order of costs for prefixes
        for j=ind   %scan prefixes from shortest one and search for corresponding suffixes
            suffix=[];  %empty vector (useful to test if a suffix was found)
            if P.trans(prefix{j}(end),prefix{j}(end))   %final state from this prefix has a loop to itself
                suffix=prefix{j}(end);
                sh_s=1; %suffix is a self-loop, associate minimum cost 1
            else
                neigh=find( P.trans(:, prefix{j}(end) ));    %states which can transit in final state of current prefix
                                                        %(we cannot use Dijkstra algorithm blindly from fs to fs, because we would get path fs, even if fs has no loop)
                if ~isempty(neigh)   %there is at least a state transiting in fs of current prefix
                    [suffix, s_cost] = find_paths(P.trans, prefix{j}(end), neigh);    %paths to each neighbor which can transit in fs
                    s_cost=s_cost+full(P.trans(neigh,prefix{j}(end)))'; %update costs, by adding costs from neighbors to the current final state
                    %suffix = suffix( find( ~cellfun('isempty',suffix)));    %elimintate empty suffixes
                    empt=find(s_cost==Inf); %eliminate empty suffixes and their costs
                    suffix(empt)=[];
                    s_cost(empt)=[];
                    if ~isempty(suffix)
                        %[sh_s ind_suf] = min( cellfun('length',suffix));    %shortest suffix (if there are more with equal length, choose the first one)
                        [sh_s ind_suf]=min(s_cost); %index of cheapest suffix (only first index is returned, if there are more with same cost)
                        %suffix=suffix{ind_suf}([2:end 1]);  %move the final state of suffix to last position; suffix is now a vector
                        suffix=suffix{ind_suf}; %first state from suffix is a final state (the one that is last state from prefix, but it will be removed from there)
                    end
                end
            end
            if ~isempty(suffix) %a suffix was found
                run_P{i}={prefix{j} suffix};    %cell containing run of P for current initial state
                r_cost(i)=p_cost(j)+sh_s;   %cost of prefix+cost of suffix
                break   %continue with next initial state
            end
        end
    end
end

%we have runs of P from each initial state (if they exist); we choose the shortest one and project it to states of Tp
%run_P=run_P( find( ~cellfun('isempty',run_P))); %remove empty runs
empt=find(r_cost==Inf); %remove empty runs
run_P(empt)=[];
r_cost(empt)=[];
if isempty(run_P)
    run_Tp=[];   %return an empty run of Tp (meaning that there is no run of Tp s.t. LTL formula is satisfied)
    return
end
% %there is at least one non-empty run of P, we want to find the one with shortest sum prefix+suffix
% for i=1:length(run_P)   %for each non-empty run of P (if there is none, the above defined empty run for Tp is returned)
%     run_length(i)=sum(cellfun('prodofsize',run_P{i}));  %prefix + suffix (to find minimum)
% end
% [r_l ind]=min(run_length);  %we need the index of run with minimum sum prefix+suffix (all runs are infinite, but this one is easiest to control)
[r_l ind]=min(r_cost);  %index of run with minimum cost for prefix+suffix
run_Tp={P.S(run_P{ind}{1}(2:end-1),1)' , P.S(run_P{ind}{2},1)'};   %projection of prefix and suffix on states of Tp
                                                                 %(states of Tp are given by the first column of matrix P.S)
                                                                 %remove first state from prefix, because it corresponds to the dummy start state of Tp (see autom_product)
                                                                 %also remove last state from prefix, because it is the first state in suffix

%eliminate possible identical successive states (due to next operators, etc.)
for i=1:2
    j=1;    %parcurge prefix/suffix of run
    while j < length(run_Tp{i})
        if run_Tp{i}(j)==run_Tp{i}(j+1)
            run_Tp{i}(j+1)=[];    %eliminate duplicate
        else
            j=j+1;  %advance if no duplicate at current position
        end
    end
end
%eliminate last state from prefix if identical with first one from suffix
try %prefix may be empty
    if run_Tp{1}(end)==run_Tp{2}(1)
        run_Tp{1}(end)=[];
    end
end
